@@ -105,22 +105,13 @@ sort_deps(Deps) ->
 droplast(L) ->
     lists:reverse(tl(lists:reverse(L))).
 
-%% @doc filtermap takes in a function that is either or both
-%% a predicate and a map, and returns the matching and valid elements.
+%% @doc wrapper around lists:filtermap/2
 -spec filtermap(F, [In]) -> [Out] when
       F :: fun((In) -> boolean() | {true, Out}),
       In :: term(),
       Out :: term().
-filtermap(F, [Hd|Tail]) ->
-    case F(Hd) of
-        true ->
-            [Hd|filtermap(F, Tail)];
-        {true,Val} ->
-            [Val|filtermap(F, Tail)];
-        false ->
-            filtermap(F, Tail)
-    end;
-filtermap(F, []) when is_function(F, 1) -> [].
+filtermap(F, In) ->
+    lists:filtermap(F, In).
 
 is_arch(ArchRegex) ->
     case re:run(get_arch(), ArchRegex, [{capture, none}, unicode]) of
@@ -982,11 +973,7 @@ get_http_vars(Scheme) ->
     Config = rebar_config:consult_file(GlobalConfigFile),
     proplists:get_value(Scheme, Config, OS).
 
--ifdef (OTP_RELEASE).
-  -if(?OTP_RELEASE >= 23).
-    -compile({nowarn_deprecated_function, [{http_uri, decode, 1}]}).
-  -endif.
--endif.
+-compile({nowarn_deprecated_function, [{http_uri, decode, 1}]}).
 
 set_httpc_options() ->
     set_httpc_options(https_proxy, get_http_vars(https_proxy)),
@@ -1142,17 +1129,9 @@ ssl_opts(ssl_verify_enabled, Url) ->
             [{verify, verify_none}]
     end.
 
--ifdef(no_customize_hostname_check).
-check_hostname_opt(Url, Opts) ->
-    #{host := Hostname} = rebar_uri:parse(rebar_utils:to_list(Url)),
-    VerifyFun = {fun ssl_verify_hostname:verify_fun/3,
-                 [{check_hostname, Hostname}]},
-    [{verify_fun, VerifyFun} | Opts].
--else.
 check_hostname_opt(_, Opts) ->
     MatchFun = public_key:pkix_verify_hostname_match_fun(https),
     [{customize_hostname_check, [{match_fun, MatchFun}]} | Opts].
--endif.
 
 -spec partial_chain(Certs) -> Res when
       Certs :: list(any()),
@@ -1225,75 +1204,5 @@ version_pad([Major, Minor, Patch | _]) ->
     {list_to_integer(Major), list_to_integer(Minor), list_to_integer(Patch)}.
 
 
--ifdef(filelib_find_source).
 find_source(Filename, Dir, Rules) ->
     filelib:find_source(Filename, Dir, Rules).
--else.
-%% Looks for a file relative to a given directory
-
--type find_file_rule() :: {ObjDirSuffix::string(), SrcDirSuffix::string()}.
-
-%% Looks for a source file relative to the object file name and directory
-
--type find_source_rule() :: {ObjExtension::string(), SrcExtension::string(),
-                             [find_file_rule()]}.
-
-keep_suffix_search_rules(Rules) ->
-    [T || {_,_,_}=T <- Rules].
-
--spec find_source(file:filename(), file:filename(), [find_source_rule()]) ->
-        {ok, file:filename()} | {error, not_found}.
-find_source(Filename, Dir, Rules) ->
-    try_suffix_rules(keep_suffix_search_rules(Rules), Filename, Dir).
-
-try_suffix_rules(Rules, Filename, Dir) ->
-    Ext = filename:extension(Filename),
-    try_suffix_rules(Rules, filename:rootname(Filename, Ext), Dir, Ext).
-
-try_suffix_rules([{Ext,Src,Rules}|Rest], Root, Dir, Ext)
-  when is_list(Src), is_list(Rules) ->
-    case try_dir_rules(add_local_search(Rules), Root ++ Src, Dir) of
-        {ok, File} -> {ok, File};
-        _Other ->
-            try_suffix_rules(Rest, Root, Dir, Ext)
-    end;
-try_suffix_rules([_|Rest], Root, Dir, Ext) ->
-    try_suffix_rules(Rest, Root, Dir, Ext);
-try_suffix_rules([], _Root, _Dir, _Ext) ->
-    {error, not_found}.
-
-%% ensuring we check the directory of the object file before any other directory
-add_local_search(Rules) ->
-    Local = {"",""},
-    [Local] ++ lists:filter(fun (X) -> X =/= Local end, Rules).
-
-try_dir_rules([{From, To}|Rest], Filename, Dir)
-  when is_list(From), is_list(To) ->
-    case try_dir_rule(Dir, Filename, From, To) of
-	{ok, File} -> {ok, File};
-	error      -> try_dir_rules(Rest, Filename, Dir)
-    end;
-try_dir_rules([], _Filename, _Dir) ->
-    {error, not_found}.
-
-try_dir_rule(Dir, Filename, From, To) ->
-    case lists:suffix(From, Dir) of
-	true ->
-	    NewDir = lists:sublist(Dir, 1, length(Dir)-length(From))++To,
-	    Src = filename:join(NewDir, Filename),
-	    case filelib:is_regular(Src) of
-		true -> {ok, Src};
-		false -> find_regular_file(filelib:wildcard(Src))
-	    end;
-	false ->
-	    error
-    end.
-
-find_regular_file([]) ->
-    error;
-find_regular_file([File|Files]) ->
-    case filelib:is_regular(File) of
-        true -> {ok, File};
-        false -> find_regular_file(Files)
-    end.
--endif.
